@@ -23,8 +23,10 @@
 #include <X11/extensions/Xvlib.h>
 #include <X11/xpm.h>
 #include <libguile.h>
+#include "util-helpers.h"
 #include "aiscm.xpm"
 
+
 #ifndef timersub
 #define timersub(a, b, result)                       \
   do {                                               \
@@ -67,23 +69,33 @@ struct window_t {
   XvImage *xv_image;
 };
 
+static struct display_t *get_display_no_check(SCM scm_self)
+{
+  return (struct display_t *)SCM_SMOB_DATA(scm_self);
+}
+
 static struct display_t *get_display(SCM scm_self)
 {
   scm_assert_smob_type(display_tag, scm_self);
-  return (struct display_t *)SCM_SMOB_DATA(scm_self);
+  return get_display_no_check(scm_self);
+}
+
+static struct window_t *get_window_no_check(SCM scm_self)
+{
+  return (struct window_t *)SCM_SMOB_DATA(scm_self);
 }
 
 static struct window_t *get_window(SCM scm_self)
 {
   scm_assert_smob_type(window_tag, scm_self);
-  return (struct window_t *)SCM_SMOB_DATA(scm_self);
+  return get_window_no_check(scm_self);
 }
 
 SCM window_destroy(SCM scm_self);
 
 SCM display_destroy(SCM scm_self)
 {
-  struct display_t *self = get_display(scm_self);
+  struct display_t *self = get_display_no_check(scm_self);
   while (!scm_is_null_and_not_nil(self->scm_windows))
     window_destroy(scm_car(self->scm_windows));
   if (self->display) {
@@ -95,7 +107,7 @@ SCM display_destroy(SCM scm_self)
 
 size_t free_display(SCM scm_self)
 {
-  struct display_t *self = get_display(scm_self);
+  struct display_t *self = get_display_no_check(scm_self);
   display_destroy(scm_self);
   scm_gc_free(self, sizeof(struct display_t), "display");
   return 0;
@@ -236,7 +248,7 @@ SCM display_set_quit(SCM scm_self, SCM scm_quit)
 
 SCM window_destroy(SCM scm_self)
 {
-  struct window_t *self = get_window(scm_self);
+  struct window_t *self = get_window_no_check(scm_self);
   if (self->xv_image) {
     XFree(self->xv_image);
     self->xv_image = NULL;
@@ -267,7 +279,7 @@ SCM window_destroy(SCM scm_self)
 
 size_t free_window(SCM scm_self)
 {
-  struct window_t *self = get_window(scm_self);
+  struct window_t *self = get_window_no_check(scm_self);
   window_destroy(scm_self);
   scm_gc_free(self, sizeof(struct window_t), "window");
   return 0;
@@ -467,7 +479,7 @@ SCM window_hide(SCM scm_self)
   return scm_self;
 }
 
-static SCM scm_convert;
+static SCM scm_convert_image;
 
 void gl_error(const char *context)
 {
@@ -475,7 +487,7 @@ void gl_error(const char *context)
   while (1) {
     GLenum err = glGetError();
     if (err == GL_NO_ERROR) break;
-    msg = scm_cons(scm_from_locale_string(gluErrorString(err)), msg);
+    msg = scm_cons(scm_from_locale_string((const char *)gluErrorString(err)), msg);
   };
   SCM str = scm_list_1(scm_string_join(msg,
                                        scm_from_locale_string(", "),
@@ -529,7 +541,7 @@ void window_paint(struct window_t *self, int x11_event)
     switch (self->io) {
       case IO_XIMAGE: {
         if (SCM_UNBNDP(self->scm_converted))
-          self->scm_converted = scm_call_3(scm_convert,
+          self->scm_converted = scm_call_3(scm_convert_image,
                                            self->scm_image,
                                            scm_from_locale_symbol("BGRA"),
                                            scm_list_2(scm_from_int(self->width),
@@ -547,7 +559,7 @@ void window_paint(struct window_t *self, int x11_event)
         break;}
       case IO_OPENGL: {
         if (SCM_UNBNDP(self->scm_converted))
-          self->scm_converted = scm_call_2(scm_convert,
+          self->scm_converted = scm_call_2(scm_convert_image,
                                            self->scm_image,
                                            scm_from_locale_symbol("RGB"));
         GLXContext context =
@@ -600,7 +612,7 @@ void window_paint(struct window_t *self, int x11_event)
         if (SCM_UNBNDP(self->scm_converted)) {
           SCM scm_offsets = scm_int_list(self->xv_image->num_planes, self->xv_image->offsets);
           SCM scm_pitches = scm_int_list(self->xv_image->num_planes, self->xv_image->pitches);
-          self->scm_converted = scm_call_5(scm_convert,
+          self->scm_converted = scm_call_5(scm_convert_image,
                                            self->scm_image,
                                            scm_car(scm_target),
                                            scm_shape,
@@ -619,7 +631,7 @@ void window_paint(struct window_t *self, int x11_event)
 
 void init_xorg(void)
 {
-  scm_convert = scm_c_public_ref("aiscm image", "convert");
+  scm_convert_image = scm_c_public_ref("aiscm image", "convert-image");
   display_tag = scm_make_smob_type("display", sizeof(struct display_t));
   window_tag = scm_make_smob_type("window", sizeof(struct window_t));
   scm_set_smob_free(display_tag, free_display);
@@ -627,18 +639,18 @@ void init_xorg(void)
   scm_c_define("IO-XIMAGE" ,scm_from_int(IO_XIMAGE));
   scm_c_define("IO-OPENGL" ,scm_from_int(IO_OPENGL));
   scm_c_define("IO-XVIDEO" ,scm_from_int(IO_XVIDEO));
-  scm_c_define_gsubr("make-display", 1, 0, 0, make_display);
-  scm_c_define_gsubr("display-shape", 1, 0, 0, display_shape);
-  scm_c_define_gsubr("display-process-events", 1, 0, 0, display_process_events);
-  scm_c_define_gsubr("display-event-loop", 2, 0, 0, display_event_loop);
-  scm_c_define_gsubr("display-quit?", 1, 0, 0, display_quit);
-  scm_c_define_gsubr("display-quit=", 2, 0, 0, display_set_quit);
-  scm_c_define_gsubr("display-destroy", 1, 0, 0, display_destroy);
-  scm_c_define_gsubr("make-window", 4, 0, 0, make_window);
-  scm_c_define_gsubr("window-show", 1, 0, 0, window_show);
-  scm_c_define_gsubr("window-title=", 2, 0, 0, window_title);
-  scm_c_define_gsubr("window-resize", 3, 0, 0, window_resize);
-  scm_c_define_gsubr("window-write", 2, 0, 0, window_write);
-  scm_c_define_gsubr("window-hide", 1, 0, 0, window_hide);
-  scm_c_define_gsubr("window-destroy", 1, 0, 0, window_destroy);
+  scm_c_define_gsubr("make-display"          , 1, 0, 0, SCM_FUNC(make_display          ));
+  scm_c_define_gsubr("display-shape"         , 1, 0, 0, SCM_FUNC(display_shape         ));
+  scm_c_define_gsubr("display-process-events", 1, 0, 0, SCM_FUNC(display_process_events));
+  scm_c_define_gsubr("display-event-loop"    , 2, 0, 0, SCM_FUNC(display_event_loop    ));
+  scm_c_define_gsubr("display-quit?"         , 1, 0, 0, SCM_FUNC(display_quit          ));
+  scm_c_define_gsubr("display-quit="         , 2, 0, 0, SCM_FUNC(display_set_quit      ));
+  scm_c_define_gsubr("display-destroy"       , 1, 0, 0, SCM_FUNC(display_destroy       ));
+  scm_c_define_gsubr("make-window"           , 4, 0, 0, SCM_FUNC(make_window           ));
+  scm_c_define_gsubr("window-show"           , 1, 0, 0, SCM_FUNC(window_show           ));
+  scm_c_define_gsubr("window-title="         , 2, 0, 0, SCM_FUNC(window_title          ));
+  scm_c_define_gsubr("window-resize"         , 3, 0, 0, SCM_FUNC(window_resize         ));
+  scm_c_define_gsubr("window-write"          , 2, 0, 0, SCM_FUNC(window_write          ));
+  scm_c_define_gsubr("window-hide"           , 1, 0, 0, SCM_FUNC(window_hide           ));
+  scm_c_define_gsubr("window-destroy"        , 1, 0, 0, SCM_FUNC(window_destroy        ));
 }
